#!/bin/sh

# This is the location of the iptables command
IPTABLES="/sbin/iptables"
TC="/sbin/tc"

INTERNET="eth0"
DMZ="eth1"
INTERNET_IP=`/sbin/ifconfig eth0 | grep inet.addr | sed "s/:/ /g" | awk '{print $3}'`
INTERNET_BCAST=`/sbin/ifconfig eth0 |grep inet.addr |sed "s/:/ /g" |awk '{print $5}'`
INTERNET_MASK=`/sbin/ifconfig eth0 |grep inet.addr |sed "s/:/ /g" |awk '{print $7}'`
echo "INTERFACE: $INTERNET IP: $INTERNET_IP BCAST: $INTERNET_BCAST MASK: $INTERNET_MASK"

DMZ="eth1"
DMZ_IP=`/sbin/ifconfig eth1 |grep inet.addr | sed "s/:/ /g" | awk '{print $3}'`
DMZ_BCAST=`/sbin/ifconfig eth1 |grep inet.addr | sed "s/:/ /g" | awk '{print $5}'`
DMZ_MASK=`/sbin/ifconfig eth1 |grep inet.addr |sed "s/:/ /g" |awk '{print $7}'`
echo "INTERFACE: $DMZ IP: $DMZ_IP BCAST: $DMZ_BCAST MASK: $DMZ_MASK"

case "$1" in
   stop)
      echo "Shutting down firewall..."
      $IPTABLES -F
      $IPTABLES -F -t mangle
      $IPTABLES -F -t nat
      $IPTABLES -X
      $IPTABLES -X -t mangle
      $IPTABLES -X -t nat

      $IPTABLES -P INPUT DROP
      $IPTABLES -P OUTPUT DROP
      $IPTABLES -P FORWARD DROP

      echo "...done"
      ;;
   status)
      echo $"Table: filter"
      iptables --list
      echo $"Table: nat"
      iptables -t nat --list
      echo $"Table: mangle"
      iptables -t mangle --list
      ;;
    restart|reload)
      sh $0 stop
      sh $0 start
      ;;

   start)
    echo "Starting Firewall..."
   
    echo ""


##--------------------------Begin Firewall---------------------------------##

#Reduce DoS'ing ability by reducing timeouts
echo 30 > /proc/sys/net/ipv4/tcp_fin_timeout
echo 2400 > /proc/sys/net/ipv4/tcp_keepalive_time
echo 0 > /proc/sys/net/ipv4/tcp_window_scaling
echo 0 > /proc/sys/net/ipv4/tcp_sack

# Allow the kernel to forward packets and prevent ipspoof
echo 1 > /proc/sys/net/ipv4/ip_forward
echo 1 > /proc/sys/net/ipv4/ip_dynaddr
for f in /proc/sys/net/ipv4/conf/*/rp_filter; do echo 1 > $f; done

# Drop packets related to cinik virus
$IPTABLES -A INPUT -p udp --dport 1978 -i $DMZ -j DROP
$IPTABLES -A INPUT -p udp --dport 2002 -i $DMZ -j DROP
$IPTABLES -A INPUT -p udp --dport 4156 -i $DMZ -j DROP

$IPTABLES -A FORWARD -p udp --dport 1978 -i $DMZ -j DROP
$IPTABLES -A FORWARD -p udp --dport 2002 -i $DMZ -j DROP
$IPTABLES -A FORWARD -p udp --dport 4156 -i $DMZ -j DROP

$IPTABLES -A INPUT -p udp --dport 1978 -i $INTERNET -j DROP
$IPTABLES -A INPUT -p udp --dport 2002 -i $INTERNET -j DROP
$IPTABLES -A INPUT -p udp --dport 4156 -i $INTERNET -j DROP

$IPTABLES -A FORWARD -p udp --dport 1978 -i $INTERNET -j DROP
$IPTABLES -A FORWARD -p udp --dport 2002 -i $INTERNET -j DROP
$IPTABLES -A FORWARD -p udp --dport 4156 -i $INTERNET -j DROP

# Accept everything coming from DMZ
$IPTABLES -A INPUT -i $DMZ -j ACCEPT
#$IPTABLES -I INPUT -s 128.198.61.11/0 -j DROP
#$IPTABLES -I FORWARD -s 128.198.61.11/0 -j DROP$

# Set up IP FORWARDing and Masquerading

#iptables -A INPUT -p tcp --dport 6779 -j ACCEPT
$IPTABLES --table nat --append POSTROUTING --out-interface $INTERNET -j MASQUERADE
$IPTABLES --append FORWARD --in-interface $DMZ -j ACCEPT

#iptables -A INPUT -p tcp --dport 6779 -j ACCEPT

# DNAT
# translate incoming ssh and www traffic to my internal hosts
iptables -t nat -A PREROUTING -p tcp --dport 21 -i $INTERNET -j DNAT --to 192.168.0.2:21
iptables -t nat -A PREROUTING -p tcp --dport 22 -i $INTERNET -j DNAT --to 192.168.0.2:22
iptables -t nat -A PREROUTING -p udp --dport 22 -i $INTERNET -j DNAT --to 192.168.0.2:22
iptables -t nat -A PREROUTING -p tcp --dport 23 -i $INTERNET -j DNAT --to 192.168.0.2:23


iptables -t nat -A PREROUTING -p udp --dport 6970:6999 -i $INTERNET -j DNAT --to 192.168.0.2
iptables -t nat -A PREROUTING -p tcp --dport 80 -i $INTERNET -j DNAT --to 192.168.0.2:80
iptables -t nat -A PREROUTING -p tcp --dport 443 -i $INTERNET -j DNAT --to 192.168.0.2:443
iptables -t nat -A PREROUTING -p tcp --dport 8080 -i $INTERNET -j DNAT --to 192.168.0.2:8080
iptables -t nat -A PREROUTING -p tcp --dport 7070 -i $INTERNET -j DNAT --to 192.168.0.2:7070
iptables -t nat -A PREROUTING -p tcp --dport 554 -i $INTERNET -j DNAT --to 192.168.0.2:554
iptables -t nat -A PREROUTING -p tcp --dport 2687 -i $INTERNET -j DNAT --to 192.168.0.2:2687
iptables -t nat -A PREROUTING -p udp --dport 8080 -i $INTERNET -j DNAT --to 192.168.0.2:8080
iptables -t nat -A PREROUTING -p udp --dport 7070 -i $INTERNET -j DNAT --to 192.168.0.2:7070
iptables -t nat -A PREROUTING -p udp --dport 554 -i $INTERNET -j DNAT --to 192.168.0.2:554
iptables -t nat -A PREROUTING -p icmp -i $INTERNET -j DNAT --to 192.168.0.2 

#Ang uncommented the following 3 lines
#iptables -t nat -A PREROUTING -p tcp -i $INTERNET -j DNAT --to 192.168.0.2
#iptables -t nat -A PREROUTING -p udp -i $INTERNET -j DNAT --to 192.168.0.2
#iptables -t nat -A PREROUTING -i $INTERNET -j DNAT --to 192.168.0.2


#$IPTABLES -t nat -A POSTROUTING -p tcp --sport 554 -o $DMZ -j SNAT --to $INTERNET_IP


$IPTABLES -A INPUT -j ACCEPT
$IPTABLES -A  FORWARD -j ACCEPT
$IPTABLES -A OUTPUT -j ACCEPT

#------End Ruleset------#

echo "...done"
echo ""

echo "--> IPTABLES firewall loaded/activated <--"


##--------------------------------End Firewall---------------------------------##
   ;;
   *)
      echo "Usage: firewall (start|stop|restart|status)"
      exit 1
esac

exit 0

