#!/usr/bin/perl

use strict;
use IO::Socket::INET;
use Time::Local;

#my $srv = "titan.uccs.edu";
#my $port = 6779;
my $confFileName = "rateif.conf";
#my $FWSettingFile = "fwsetting.dat";
#my $IPTABLES = "/sbin/iptables";
#my $alarmTime = 1;

my ($srv,$port,$FWSettingFile,$IPTABLES,$alarmTime);
my(@levelList, @ipList);
my $ipListIndex = 0;
my($ruleString, $fwdRuleString);
# The column names of ipList
my $IP = 0;
my $LEVEL = 1;
my $BLOCKLVL = 2;
my $REMTIME = 3;

# The column names of levelList
my $LVLNAME = 0;
my $RATE = 1;
my $TIME = 2;

sub parseMesg {
	my($mesg) = @_;
	my($ipAddr, $tmpMesg);
	$tmpMesg = $mesg;
	#$ipAddr = $mesg;
	#( $i1, $i2, $i3, $i4) 
	#if($mesg =~ m/.+FLOOD DETECTED([^\d]+)(\d+)\.(\d+)\.(\d+).(\d+).+/) {
	if($mesg =~ m/.*End\s+of.*/) {
		return -1;
	}
	elsif($mesg =~ m/(\d+)\.(\d+)\.(\d+).(\d+)(\/\d+)/) {
	#$ipAddr = $2.".".$3.".".$4.".".$5;
	$ipAddr = $1.".".$2.".".$3.".".$4.$5;
	return $ipAddr;
	}
	elsif($mesg =~ m/(\d+)\.(\d+)\.(\d+).(\d+)/) {
	#$ipAddr = $2.".".$3.".".$4.".".$5;
	$ipAddr = $1.".".$2.".".$3.".".$4;
	return $ipAddr;
	}
	else {
		return -1;
	}
}

sub convertIntoSecs {
	my($time) = @_;
	if($time == -1) {
	return $time;
	}
	my($day,$hr,$min, $sec) = split(/:/g,$time);
	my $toSecs = $day * 86400 + $hr * 3600 + $min * 60 + $sec;
	return $toSecs;
}

sub readConfFile{
	if(!(-T $confFileName)) {
		print "ERROR: Configuration file not found!\n";
		exit(1);
	}
	print "Reading the configuration file...\n";
	my($el, $i);
	open(CONFFILE,$confFileName);
	my @config = <CONFFILE>;
	@config = grep(!/(\S*)#(.*)/, @config);
	@config = grep(!/^\n$/, @config);
	#print @config;
	my @settingsList = grep(!/(.+)(\S+)(.+)(\S+)(\d+):(\d+):(\d+):(\d+)/, @config);
	@config = grep(/(.+)(\S+)(.+)(\S+)(\d+):(\d+):(\d+):(\d+)/, @config);
	#print @config;
	@config = reverse @config;
	#print @settingsList;	

	my $stng;
	
	foreach $stng (@settingsList) {
		chomp $stng;
		$stng =~ m/(\w+)(\s*)(.+)/;
		my $setting = $1;
		my $value = $3;
		
		
		#print "Setting: $setting\tValue: $value\n";
		#print "$1\t$2\t$3\t$4\n\n";
		if($setting eq "SERVER") {
			$srv = $value;
		}
		elsif($setting eq "PORT") {
			$port = int($value);
		}
		elsif($setting eq "SETTINGSFILE") {
			$FWSettingFile = $value;
		}
		elsif($setting eq "IPTABLES") {
			$IPTABLES = $value;
		}
		elsif($setting eq "ALARMTIME") {
			$alarmTime = int($value);
		}
	}
	
print "$srv\n$port\n$confFileName\n$FWSettingFile\n$IPTABLES\n$alarmTime\n";
	
	$i = 0;
	foreach $el (@config) {
		($levelList[$i][0], $levelList[$i][1], $levelList[$i][2]) = split(/\s+/g, $el);
		$i++; 
	}
	print "The levels\n";
	for($i= 0; $i <= $#levelList; $i++){
		$levelList[$i][2] = convertIntoSecs($levelList[$i][2]);
 print $levelList[$i][0]," ",$levelList[$i][1]," ",$levelList[$i][2],"\n";
	}

}

sub restoreFWSettings {
	if(!(-T $FWSettingFile)) {
		return;
	}
		print "Loading last known IP list...\n";
		open(FWSETTING,$FWSettingFile);
		my @tmpList = <FWSETTING>;
		my($el, $i);
		$i=0;
	foreach $el (@tmpList) {
	($ipList[$i][0],$ipList[$i][1],$ipList[$i][2],$ipList[$i][3]) = split(/\s+/,$el);
		$i++;
	}	
	
}

sub deleteRule{
	my($index) = int(@_);
	open(RULEFILE,">> rulefile");
	print RULEFILE $ruleString;
	print RULEFILE $fwdRuleString;
	system($ruleString);
	system($fwdRuleString);
	print $ruleString;
	print $fwdRuleString;
	close RULEFILE;
}

sub addRule{
	my($index) = @_;
	$index = int($index);
	my($rate);
	open(RULEFILE,">> rulefile");
	#if(int($levelList[int($ipList[$index][$LEVEL])][$RATE]) == -1) {
	#	$rate = "drop";
	#}
	#else {
	#	$rate = int($levelList[int($ipList[$index][$LEVEL])][$RATE]); 
		
	#}
	#print RULEFILE "Adding rule: ",$ipList[$index][$IP]," L",$ipList[$index][$LEVEL]," ",$rate,"\n ";
	
	print RULEFILE "$IPTABLES -I INPUT -s ",$ipList[$index][$IP]," -j ","level",$ipList[$index][$LEVEL],"\n";
	my $addRule1 = "$IPTABLES -I INPUT -s ".$ipList[$index][$IP]." -j "."level".$ipList[$index][$LEVEL]."\n";
	system($addRule1);
	print $addRule1;
	$addRule1 =~ s/INPUT/FORWARD/g;
	system($addRule1);
	print $addRule1;
	print RULEFILE $addRule1;
	
	close RULEFILE;
}

sub createDeleteRule {
	my($index) = @_;
	$index = int($index);
	my($rate);
	if(int($levelList[int($ipList[$index][$LEVEL])][$RATE]) == -1) {
		$rate = "drop";
	}
	else {
		$rate = int($levelList[int($ipList[$index][$LEVEL])][$RATE]); 
		
	}
	#$ruleString =  "Delete rule: ".$ipList[$index][$IP]." L".$ipList[$index][$LEVEL]." ".$rate."\n ";
	# system("Firewall rule");
	$ruleString =  "$IPTABLES -D INPUT -s ".$ipList[$index][$IP]." -j "."level".$ipList[$index][$LEVEL]."\n";
	#system($ruleString);
	$fwdRuleString =  "$IPTABLES -D FORWARD -s ".$ipList[$index][$IP]." -j "."level".$ipList[$index][$LEVEL]."\n";
	#system($ruleString);
}

sub writeToFile {

       # my $time = localtime($sec, $min, $hour, $mday, $mon, $year);
	my $logTime = localtime();
    	
	open(LOG,">>logfile");
        print LOG "$logTime\n";
	for(my $i = 0; $i <= $#ipList; $i++) {
		print LOG $ipList[$i][$IP]," L",$ipList[$i][$LEVEL]," L",$ipList[$i][$BLOCKLVL]," ",$ipList[$i][$REMTIME],"\n";
	}
	close LOG;
	
}

sub checkExpire {
	#print "*";
	my($i);
	for($i = 0; $i < $ipListIndex; $i++) {
		$ipList[$i][$REMTIME]=int($ipList[$i][$REMTIME]) -  $alarmTime;
		if(int($ipList[$i][$REMTIME]) <= 0) {
			if(int($ipList[$i][$LEVEL]) == $#levelList) {
				splice(@ipList,$i,1,);
				$ipListIndex--;
			}
			else {
				open(RULEFILE,">> rulefile");
				print RULEFILE "Time out:\n";
				close RULEFILE;
				if(int($levelList[int($ipList[$i][$LEVEL])][$RATE]) == -1){
				#Delete the rule only
				createDeleteRule($i);
				$ipList[$i][$LEVEL]=$#levelList;
				$ipList[$i][$REMTIME] = $levelList[$#levelList][$TIME];
				deleteRule;
				}
				else {
					createDeleteRule($i);
					$ipList[$i][$LEVEL]++;
					if(int($ipList[$i][$LEVEL]) == $#levelList) {
						splice(@ipList,$i,1,);
						$ipListIndex--;
						deleteRule;
					}
					else {
						$ipList[$i][$REMTIME] = $levelList[int($ipList[$i][$LEVEL])][$TIME];
						addRule($i);
						deleteRule;
						
					}
				}
			}
		}
		
	}
	writeToFile();
	alarm $alarmTime;
}

sub initializeRules{
      my($i, $initRules);
system("$IPTABLES -F\n");
system("$IPTABLES -F -t mangle\n");
system("$IPTABLES -F -t nat\n");
system("$IPTABLES -X\n");
system("$IPTABLES -X -t mangle\n");
system("$IPTABLES -X -t nat\n");
print("$IPTABLES -F\n");
print("$IPTABLES -F -t mangle\n");
print("$IPTABLES -F -t nat\n");
print("$IPTABLES -X\n");
print("$IPTABLES -X -t mangle\n");
print("$IPTABLES -X -t nat\n");

	open(RULEFILE,">> rulefile");
	for($i=0; $i <= $#levelList-1; $i++) { 	
        	print RULEFILE "$IPTABLES -N level$i\n";
		$initRules = "$IPTABLES -N level$i\n";
		system($initRules);
		print $initRules;
		if(int($levelList[$i][$RATE]) != -1) {
		      print RULEFILE "$IPTABLES -A level$i -m limit --limit ",int($levelList[$i][$RATE]),"/s  -j ACCEPT\n";
		     $initRules = "$IPTABLES -A level$i -m limit --limit ".int($levelList[$i][$RATE])."/s  -j ACCEPT\n";
		     system($initRules);
		    print $initRules;
		}
        	print RULEFILE "$IPTABLES -A level$i -j DROP\n\n";
        	$initRules =  "$IPTABLES -A level$i -j DROP\n";
		system($initRules);
		print $initRules;
      }
	close RULEFILE;
}

sub processIP{
	my($ip) = @_;
	my($i,$index);
	$index = -1;
	for($i=0; $i <= $#ipList; $i++) {
		if($ipList[$i][0] eq $ip) {
			$index = $i;
		} 
	}

	if($index != -1) {
		# if the block level is not -1, it means that the ip has been or is currently blocked.
		if(int($ipList[$index][$BLOCKLVL]) != -1) {
			if(int($ipList[$index][$BLOCKLVL]) == 0) {
				# Add a rule that blocks
				$ipList[$index][$LEVEL] = 0;
				$ipList[$index][$REMTIME] = $levelList[0][$TIME];
				addRule($index);
			}
			else {
				# Do nothing, let it stay blocked
				$ipList[$index][$LEVEL] = int($ipList[$index][$BLOCKLVL]) - 1;
				$ipList[$index][$BLOCKLVL] = int($ipList[$index][$LEVEL]);
				$ipList[$index][$REMTIME] = $levelList[int($ipList[$index][$LEVEL])][$TIME];
				addRule($index);
			}			
		}
		else {
			createDeleteRule($index);
			$ipList[$index][$LEVEL] = int($ipList[$index][$LEVEL]) - 1;
			$ipList[$index][$REMTIME] = $levelList[int($ipList[$index][$LEVEL])][$TIME];
			#Firewall functions come here
			if($levelList[int($ipList[$index][$LEVEL])][$RATE] == -1) {
		$ipList[$index][$BLOCKLVL] = int($ipList[$index][$LEVEL]);
			}
			addRule($index);	
			deleteRule;
			
		}
	}
	else {
		#$ipList[$ipListIndex]
		$ipList[$ipListIndex][$IP] = $ip;
		$ipList[$ipListIndex][$LEVEL] = (int($#levelList) - 1);
		#print "Level:$ipList[$ipListIndex][$LEVEL] \n";
		$ipList[$ipListIndex][$BLOCKLVL] = -1;
		$ipList[$ipListIndex][$REMTIME] = $levelList[int($ipList[$ipListIndex][$LEVEL])][$TIME];
		addRule($ipListIndex);	
		$ipListIndex++;
		#Firewall functions come here
		
	}
		
}


	$SIG{ALRM} = \&checkExpire;

#if(fork() > 0) {
#}
#else {
readConfFile();
initializeRules;
#restoreFWSettings();

alarm $alarmTime;
my $merSock = new IO::Socket::INET(LocalHost => $srv,
                                 LocalPort => $port,
                                 Proto => 'tcp',
                                 Listen => 5,
                                 Reuse => 1);                                   
        die "Couldn't create a listening socket!" unless $merSock;

      my($newSock, $ipAddr);
        while($newSock = $merSock->accept()) {
		my $mesg = <$newSock>;
		chomp $mesg;
		close($newSock);
		$ipAddr = parseMesg($mesg);
		print "IPADDR: $ipAddr\n";
		#exit 1;		
		#print "Recvd IP: $ipAddr\n";
		if(int($ipAddr) != -1) {
		processIP($ipAddr);
		}
	}
#}
